// editPaths.js
var pathChangesMade = false;

var pathsError = false;

// this is used to hold the ServiceGroup Entry being edited/created
var tmpServiceGroupEntry = new Array();

var pathTypeOptions = new Array();
pathTypeOptions['firewallToFirewall'] = 0;
pathTypeOptions['firewallToInternal'] = 1;
pathTypeOptions['firewallToExternal'] = 2;
pathTypeOptions['firewallToIPSec'] = 3;
pathTypeOptions['firewallToDMZ'] = 4;
pathTypeOptions['firewallToDialin'] = 5;
pathTypeOptions['internalToFirewall'] = 6;
pathTypeOptions['internalToInternal'] = 7;
pathTypeOptions['internalToExternal'] = 8;
pathTypeOptions['internalToIPSec'] = 9;
pathTypeOptions['internalToDMZ'] = 10;
pathTypeOptions['internalToDialin'] = 11;
pathTypeOptions['internalBroadcast'] = 12;
pathTypeOptions['externalToFirewall'] = 13;
pathTypeOptions['externalToInternal'] = 14;
pathTypeOptions['externalToExternal'] = 15;
pathTypeOptions['externalToIPSec'] = 16;
pathTypeOptions['externalToDMZ'] = 17;
pathTypeOptions['externalToDialin'] = 18;
pathTypeOptions['externalBroadcast'] = 19;
pathTypeOptions['ipsecToFirewall'] = 20;
pathTypeOptions['ipsecToInternal'] = 21;
pathTypeOptions['ipsecToExternal'] = 22;
pathTypeOptions['ipsecToIPSec'] = 23;
pathTypeOptions['ipsecToDMZ'] = 24;
pathTypeOptions['ipsecToDialin'] = 25;
pathTypeOptions['ipsecBroadcast'] = 26;
pathTypeOptions['dmzToFirewall'] = 27;
pathTypeOptions['dmzToInternal'] = 28;
pathTypeOptions['dmzToExternal'] = 29;
pathTypeOptions['dmzToIPSec'] = 30;
pathTypeOptions['dmzToDMZ'] = 31;
pathTypeOptions['dmzToDialin'] = 32;
pathTypeOptions['dmzBroadcast'] = 33;
pathTypeOptions['dialinToFirewall'] = 34;
pathTypeOptions['dialinToInternal'] = 35;
pathTypeOptions['dialinToExternal'] = 36;
pathTypeOptions['dialinToIPSec'] = 37;
pathTypeOptions['dialinToDMZ'] = 38;
pathTypeOptions['dialinToDialin'] = 39;
pathTypeOptions['dialinBroadcast'] = 40;

// define those path tags that only have i=, limitFrom= attributes
var incomingPaths = /^(internalBroadcast|externalBroadcast|ipsecBroadcast|dmzBroadcast|dialinBroadcast|internalToFirewall|externalToFirewall|ipsecToFirewall|dmzToFirewall|dialinToFirewall)$/;

// define those path tags that only have o=, limitTo= attributes
var outgoingPaths = /^(firewallToInternal|firewallToExternal|firewallToIPSec|firewallToDMZ|firewallToDialin|firewallToFirewall)$/;

// define those path tags that have all i=, o=, limitFrom=, limitTo= attributes
var bothPaths = /^(internalToInternal|internalToExternal|internalToIPSec|internalToDMZ|internalToDialin|externalToExternal|externalToInternal|externalToIPSec|externalToDMZ|externalToDialin|ipsecToInternal|ipsecToExternal|ipsecToIPSec|ipsecToDMZ|ipsecToDialin|dmzToInternal|dmzToExternal|dmzToIPSec|dmzToDMZ|dmzToDialin|dialinToInternal|dialinToExternal|dialinToIPSec|dialinToDMZ|dialinToDialin)$/;

// define those path tags that are valid to have the ip= option specified.
var ipPaths = /^(externalToFirewall|internalToFirewall|ipsecToFirewall|dmzToFirewall|dialinToFirewall)$/;

var pathActionOptions = new Array();
pathActionOptions['ACCEPT'] = 0;
pathActionOptions['DROP No LOG'] = 1;
pathActionOptions['REJECT'] = 2;
pathActionOptions['REDIRECT'] = 3;
pathActionOptions['DNAT'] = 4;
pathActionOptions['QUEUE'] = 5;

var pathDNATZoneOptions = new Array();
pathDNATZoneOptions['internal'] = 0;
pathDNATZoneOptions['ipsec'] = 1;
pathDNATZoneOptions['dmz'] = 2;
pathDNATZoneOptions['dialin'] = 3;

function pathTypeSelectOptions()
{
  var output = '';

  for (var option in pathTypeOptions)
  {
    if (option.length > 0)
    {
      output += '    <option value="' + option + '">' + option + '</option>\n';
    }
  }

  return output;
}

function pathActionSelectOptions()
{
  var output = '';

  for (var option in pathActionOptions)
  {
    if (option.length > 0)
    {
      output += '    <option value="' + option + '">' + option + '</option>\n';
    }
  }

  return output;
}

function pathDNATZoneSelectOptions()
{
  var output = '';

  for (var option in pathDNATZoneOptions)
  {
    if (option.length > 0)
    {
      output += '    <option value="' + option + '">' + option + '</option>\n';
    }
  }

  return output;
}

var availableNetworksOptions = new Array();
var availableNetworksMappingsForward = new Array();
var availableNetworksMappingsReverse = new Array();

function updateAvailableNetworksOptions()
{
  var counter = 0;

  for (var networkIndex in availableNetworksOptions)
  {
    delete availableNetworksOptions[networkIndex];  // empty the array.
  }
  for (var networkIndex in availableNetworksMappingsForward)
  {
    delete availableNetworksMappingsForward[networkIndex];  // empty the array.
  }
  for (var networkIndex in availableNetworksMappingsReverse)
  {
    delete availableNetworksMappingsReverse[networkIndex];  // empty the array.
  }
  for (var n=0; n < dataObj.networks.length; n++)
  {
    if (dataObj.networks[n] != undefined)
    {
      var entry = dataObj.networks[n].host + (dataObj.networks[n].comment.length > 0 ? ' (' + dataObj.networks[n].comment + ')' : '');
      availableNetworksOptions[entry] = counter++;
      // keep track of the network number.
      availableNetworksMappingsForward[n] = entry;
      availableNetworksMappingsReverse[entry] = n;
    }
  }
}

function availableNetworksSelectOptions()
{
  var output = '';

  for (var option in availableNetworksOptions)
  {
    if (option.length > 0)
    {
      output += '    <option value="' + option + '">' + option + '</option>\n';
    }
  }

  return output;
}

var availableServiceOptions = new Array();

function updateAvailableServiceOptions()
{
  var counter = 0;

  for (var serviceIndex in availableServiceOptions)
  {
    delete availableServiceOptions[serviceIndex];  // empty the array.
  }
  for (var n in dataObj.services['servicesOrder'])
  {
    if (dataObj.services['servicesOrder'][n] != undefined)
    {
      availableServiceOptions[dataObj.services['servicesOrder'][n]] = counter++;
    }
  }
}

function availableServiceSelectOptions()
{
  var output = '';

  for (var option in availableServiceOptions)
  {
    if (option.length > 0)
    {
      output += '    <option value="' + option + '">' + option + '</option>\n';
    }
  }

  return output;
}

// I really need to make the list of available interfaces be limited to just those defined in the zone
// the path is wanting to work with.  There should be a method that will be called with the path type name
// and the in/out direction to work with and it then figures out the correct zone and proceeds to update the
// in/out select list with the available interfaces.

function updateAvailableInterfacesOptions(path, input)
{
  var zone = "";
  var config = document.configF;

  // verify the input direction is valid.
  if (input == 'i')
  {
    // verify the path exists in incoming and/or both paths.
    if (incomingPaths.test(path) || bothPaths.test(path))
    {
      // break apart the specified path and figure out the zone to work with
      var normalPattern = /^(.+)To(.+)$/;
      var broadcastPattern = /^(.+)Broadcast$/;

      var result = path.match(normalPattern);
      if (result != null)
      {
        zone = result[1];  // The incoming zone
      }
      else
      {
        result = path.match(broadcastPattern);
        if (result != null)
        {
          zone = result[1];  // The incoming zone
        }
        else
        {
          alert('updateAvailableInterfacesOptions(' + path + ', ' + input + ') path not found!');
          return;
        }
      }
    }
    else
    {
      alert('updateAvailableInterfacesOptions(' + path + ', ' + input + ') is invalid!');
      return;
    }

    // empty the in select box.
    config.i.options.length = 0;

    if (path != 'firewallToFirewall')
    {
      config.i.options[0] = new Option('* - All interfaces', '*');
    }
  }
  else if (input == 'o')
  {
    // verify the path exists in outgoing and/or both paths.
    if (outgoingPaths.test(path) || bothPaths.test(path))
    {
      // break apart the specified path and figure out the zone to work with
      var normalPattern = /^(.+)To(.+)$/;

      var result = path.match(normalPattern);
      if (result != null)
      {
        zone = result[2];  // The outgoing zone
      }
      else
      {
        alert('updateAvailableInterfacesOptions(' + path + ', ' + input + ') path not found!');
        return;
      }
    }
    else
    {
      alert('updateAvailableInterfacesOptions(' + path + ', ' + input + ') is invalid!');
      return;
    }

    // empty the out select box.
    config.o.options.length = 0;

    if (path != 'firewallToFirewall')
    {
      config.o.options[0] = new Option('* - All interfaces', '*');
    }
  }
  else if (input == 'DNAT')
  {
    // the zone to get interfaces for is sent in as the 'path'.
    zone = path;
    input = "dnatInterface";

    config.dnatInterface.options.length = 0;
  }
  else
  {
    alert('updateAvailableInterfacesOptions(' + path + ', ' + input + ') is invalid!');
    return;
  }

  zone = zone.toLowerCase();  // make sure we have the lowercase version of the zone.

  var zones = new Array();
  if (zone == 'firewall')
  {
    zones = new Array ("internal", "external", "ipsec", "dmz", "dialin");
  }
  else
  {
    zones[0] = zone;
  }

  // zero out the select box first
  // eval("config." + input + ".options.length = 0;");

  var counter = (input == 'dnatInterface' || path == 'firewallToFirewall' ? 0 : 1);
  for (var zoneIndex in zones)
  {
    var tmpZone = zones[zoneIndex];
    for (var alias in dataObj.zones[tmpZone])
    {
      try
      {
        eval("config." + input + ".options[" + counter++ + "] = new Option('" + alias + (!dataObj.zones[tmpZone][alias].active || (tmpZone != "external" && dataObj.config.zones[tmpZone] == 0) ? " *" : "") + "', '" + alias + "');");
      }
      catch (exception)
      {
        alert(exception);
        return;
      }
    }
  }
  if (zone == 'firewall')
  {
    // we don't want the interface selected by default.
    eval("config." + input + ".selectedIndex = -1;");
  }
}

// use this to find out the index of the interface in the given interface select box.
function selectPathInterface(field, name)
{
  var found = false;
  for (var i=0; i < field.options.length; i++)
  {
    if (typeof name == "object" && name instanceof Array)
    {
      for (var j in name)
      {
        if (field.options[i].value == name[j])
        {
          field.options[i].selected = true;
          found = true;
        }
      }
    }
    else
    {
      if (field.options[i].value == name)
      {
        field.options[i].selected = true;
        field.selectedIndex = i;
        return;
      }
    }
  }
  if (found)
  {
    return;
  }
  field.selectedIndex = -1;
}

function selectPathHosts(field, entries)
{
  field.selectedIndex = -1;
  if (entries.length == 0)
  {
    return;
  }
  if (field.options.length == 0)
  {
    return;
  }
  // make sure we deselect any previous selections.
  for (var i=0; i < field.options.length; i++)
  {
    field.options[i].selected = false;
  }
  // now select those entries currently specified in the entries array.
  for (var j=0; j < entries.length; j++)
  {
    if (entries[j] == undefined) continue;
    for (var i=0; i < field.options.length; i++)
    {
      //alert("j = '" + j + "', entries[j] = '" + entries[j] + "', field.options[" + i + "].value = '" + field.options[i].value + "', avalableNetworksMappingsForward[" + entries[j] + "] = '" + availableNetworksMappingsForward[entries[j]] + "'");
      if (field.options[i].value == availableNetworksMappingsForward[entries[j]])
      {
        field.options[i].selected = true;
        break;
      }
    }
  }
}

function selectPathService(entries)
{
  var f = document.configF;

  f.service.selectedIndex = -1;
  if (entries.length == 0)
  {
    return;
  }
  if (f.service.options.length == 0)
  {
    return;
  }
  for (var i=0; i < f.service.options.length; i++)
  {
    f.service.options[i].selected = false;
  }
  for (var j=0; j < entries.length; j++)
  {
    if (entries[j] == undefined) continue;
    for (var i=0; i < f.service.options.length; i++)
    {
      if (f.service.options[i].value == entries[j])
      {
        f.service.options[i].selected = true;
        break;
      }
    }
  }
}

function togglePathInterfaceFields(input, value, path)
{
  var f = document.configF;

  if (input == 'i')
  {
    f.i.disabled = value;
    f.limitFrom.disabled = value;
    displaySelectButtons('limitFrom', value);
    if (value == false)
    {
      updateAvailableInterfacesOptions(path, input);
    }
    else
    {
      f.i.options.length = 0;
    }
    var result = path.match(ipPaths);
    if (result != null)
    {
      f.ip.disabled = value;
    }
    else
    {
      f.ip.disabled = true;
    }
  }
  else
  {
    f.o.disabled = value;
    f.limitTo.disabled = value;
    displaySelectButtons('limitTo', value);
    if (value == false)
    {
      updateAvailableInterfacesOptions(path, input);
    }
    else
    {
      f.o.options.length = 0;
    }
  }
}

function togglePathRedirectFields(value)
{
  var f = document.configF;

  f.redirectPort.disabled = value;
}

function togglePathDNATFields(value)
{
  var f = document.configF;

  f.dnatDestination.disabled = value;
  f.dnatPort.disabled = value;
  f.dnatZone.disabled = value;
  f.dnatInterface.disabled = value;
  if (value == false)
  {
    if (f.dnatZone.selectedIndex == -1)
    {
      f.dnatZone.selectedIndex = 0;
    }
    updateAvailableInterfacesOptions(f.dnatZone.options[f.dnatZone.selectedIndex].value, 'DNAT');
  }
}

function handlePathActionChange()
{
  var f = document.configF;
  var action = f.action.options[f.action.selectedIndex].value;
  if (action == 'REDIRECT' || action == 'DNAT')
  {
    if (action == 'REDIRECT')
    {
      togglePathRedirectFields(false);
      togglePathDNATFields(true);
    }
    else if (action == 'DNAT')
    {
      togglePathRedirectFields(true);
      togglePathDNATFields(false);
    }
    togglePathLogFields(false);
  }
  else
  {
    togglePathRedirectFields(true);
    togglePathDNATFields(true);
    if (action == 'DROP No LOG')
    {
      togglePathLogFields(true);
    }
    else
    {
      togglePathLogFields(false);
    }
  }
}

function handleInterfaceChange(field, path)
{
  // make sure that the user hasn't selected * and any other interface.
  var errorCondition = false;
  if (field.options.length == 0)
  {
    return true;  // there are no interfaces for this path to select.
  }
  // make sure they selected an interface if it is required.
  if (field.selectedIndex == -1)
  {
    // make sure it isn't required.
    if (field.name == 'i')
    {
      if (bothPaths.test(path) || incomingPaths.test(path))
      {
        alert("You must select an incoming interface!");
        return false;
      }
    }
    else if (field.name == 'o')
    {
      if (bothPaths.test(path) || outgoingPaths.test(path))
      {
        alert("You must select an outgoing interface!");
        return false;
      }
    }
  }
  var starSelected = (field.options[0].value == '*' && field.options[0].selected ? 1 : 0);
  for (var i=(field.options[0].value == '*' ? 1 : 0); i < field.options.length; i++)
  {
    if (field.options[i].selected && starSelected)
    {
      errorCondition = true;
      field.options[i].selected = false;
    }
  }
  if (errorCondition)
  {
    alert("You can not select interfaces when you selected the '*' interface!");
    return false;
  }
  return true;
}

function togglePathLogFields(value)
{
  var f = document.configF;
  var action = f.action.options[f.action.selectedIndex].value;

  if (action == 'DROP No LOG')
  {
    f.log.disabled = true;
  }
  else
  {
    f.log.disabled = false;
  }
  if (value == false && f.log.checked)
  { // we are logging so enable the comment and level entries
    f.logComment.disabled = false;
    f.logLevel.disabled = false;
  }
  else
  { // disable the comment and level entries
    f.logComment.disabled = true;
    f.logLevel.disabled = true;
  }
}

var pathTypeHandler = " onChange=\"var f=document.configF; var tmpType = f.type.options[f.type.selectedIndex].value; if(bothPaths.test(tmpType)) { togglePathInterfaceFields(\'i\', false, tmpType); togglePathInterfaceFields(\'o\', false, tmpType); f.i.focus(); } else { if (incomingPaths.test(tmpType)) { togglePathInterfaceFields(\'i\', false, tmpType); togglePathInterfaceFields(\'o\', true, tmpType); f.i.focus(); } else { if (outgoingPaths.test(tmpType)) { togglePathInterfaceFields(\'i\', true, tmpType); togglePathInterfaceFields(\'o\', false, tmpType); f.o.focus(); } } }\"";

var actionHandler = " onChange=\"handlePathActionChange();\"";
var dnatZoneHandler = " onChange=\"var f=document.configF; updateAvailableInterfacesOptions(f.dnatZone.options[f.dnatZone.selectedIndex].value, 'DNAT');\"";
var logHandler = " onclick=\"var f=document.configF; togglePathLogFields(!f.log.checked);\"";

var interfaceHandler = " onChange=\"handleInterfaceChange(this);\"";

// define the <paths> arguments
nameLookups['paths'] = new Array();
nameLookups['serviceGroup'] = new Array();

createLookupEntry('serviceGroup', 'name', 'Name', 'text', '', 'Specify the name of the serviceGroup you are creating.');
createLookupEntry('serviceGroup', 'comment', 'Comment', 'textarea', '', 'Specify any comment about this serviceGroup that will help remind you of what it is doing.');
createLookupEntry('serviceGroup', 'active', 'Active', 'bool', '', 'Is this serviceGroup active?');

createLookupEntry('paths', 'type', 'Type', 'select', pathTypeSelectOptions, 'Specify the type of path you wish to implement.', pathTypeHandler);
createLookupEntry('paths', 'i', 'Incoming Interface', 'multiselect', '', 'Specify the interface(s) this traffic is to come in.  <b>*</b> means all interfaces in the specified from zone.', interfaceHandler);
createLookupEntry('paths', 'limitFrom', 'limit From', 'multiselect', availableNetworksSelectOptions, 'Specify the source(s) to limit this traffic from.');
createLookupEntry('paths', 'o', 'Outgoing Interface', 'multiselect', '', 'Specify the interface(s) this traffic is to go out.  <b>*</b> means all interfaces in the specified to zone.', interfaceHandler);
createLookupEntry('paths', 'limitTo', 'limit To', 'multiselect', availableNetworksSelectOptions, 'Specify the destination(s) to limit this traffic to.');
createLookupEntry('paths', 'comment', 'Comment', 'textarea', '', 'Specify any comment about this path that will help remind you of what it is doing.');
createLookupEntry('paths', 'active', 'Active', 'bool', '', 'Is this path active?');
createLookupEntry('paths', 'action', 'Action', 'select', pathActionSelectOptions, 'Specify the action to take for this path.  The default is <b>Accept</b>.<br /><br /><b>DROP No LOG</b> will drop the packet without logging it, by default the dropped packets are logged.<br /><br /><b>REJECT</b> will drop the packet after notifying the sender.<br /><br /><b>REDIRECT</b> will send the packet to another port on the firewall.<br /><br /><b>DNAT</b> will send the packet to another host and port.<br /><br /><b>QUEUE</b> will allow the packet to be processed by a program running on the firewall.  This is normally used for snort-inline.', actionHandler);
createLookupEntry('paths', 'redirectPort', 'Port', 'text', '', 'Specify the port to REDIRECT this traffic to.  If empty, it goes to the port(s) defined by the &lt;service/&gt;(s).');
createLookupEntry('paths', 'dnatDestination', 'Destination', 'select', availableNetworksSelectOptions, 'Specify the destination to send this traffic to.');
createLookupEntry('paths', 'dnatPort', 'Port', 'text', '', 'Specify the port to DNAT this traffic to.  If empty, it goes to the port(s) defined by the &lt;service/&gt;(s).');
createLookupEntry('paths', 'dnatZone', 'Zone', 'select', pathDNATZoneSelectOptions, 'Specify the zone this DNATted traffic is to go out.', dnatZoneHandler);
createLookupEntry('paths', 'dnatInterface', 'Outgoing Interface', 'select', '', 'Specify the interface this DNATted traffic is to go out.  It must exist in the select Zone.');
createLookupEntry('paths', 'log', 'Log', 'bool', '', 'Is this path LOGged?', logHandler);
createLookupEntry('paths', 'logComment', 'Comment', 'text', '', 'Specify the comment you want logged.');
createLookupEntry('paths', 'logLevel', 'Level', 'select', logLevelSelectOptions, 'Specify the syslog level to log at.');
createLookupEntry('paths', 'ip', 'IP', 'select+clear', availableNetworksSelectOptions, 'Specify the IP Address the traffic has to be destined for on the firewall so we can DNAT, REDIRECT, etc. and handle the cross firewall scenarios.');
createLookupEntry('paths', 'service', 'Service', 'multiselect', availableServiceSelectOptions, 'Specify the &lt;service&gt;(s) this path is to filter.');

function processEditServiceGroupClick()
{
  var config = document.configF;

  if (config.serviceGroup.selectedIndex == -1)
  {
    alert("You must select a ServiceGroup to Edit!");
  }
  else
  {
    var serviceGroup = config.serviceGroup.options[config.serviceGroup.selectedIndex].value;
    editServiceGroupsEntry(serviceGroup);
  }
}

function processDeleteServiceGroupClick()
{
  var config = document.configF;

  if (config.serviceGroup.selectedIndex == -1)
  {
    alert("You must select a ServiceGroup to Delete!");
  }
  else
  {
    var serviceGroup = config.serviceGroup.options[config.serviceGroup.selectedIndex].value;
    deleteServiceGroupsEntry(serviceGroup);
  }
}

function processMoveServiceGroupClick()
{
  var config = document.configF;

  if (config.serviceGroup.selectedIndex == -1)
  {
    alert("You must select a ServiceGroup to Move!");
  }
  else
  {
    var serviceGroup = config.serviceGroup.options[config.serviceGroup.selectedIndex].value;
    moveServiceGroupsEntry(serviceGroup);
  }
}

function displayServiceGroups()
{
  var content = getContent();

  if (! verifyChangesSaved())
  {
    return;
  }
  entryChangesMade = false;

  // make sure we have the latest available networks and services.
  updateAvailableNetworksOptions();
  updateAvailableServiceOptions();

  displayStatus("Editing Paths...<br>\nIndividual &lt;path&gt;'s are grouped by &lt;serviceGroup&gt; tags thus enabling you to designate that groups of paths are dealing with a specific service.  There is nothing saying you have to use the same service in the paths, it is just for human readability and a sense of order and grouping.<br><br>\nSelect the ServiceGroup from the list to Edit or Delete, or click the <b>Create</b> button to create a new entry.", "info");

  // build up the form html to display the dataObj.services info in.
  var html = "";
  html += '  <center><b>ServiceGroups</b><br>\n';
  html += '  <select name="serviceGroup">\n';

  for (var serviceGroupIndex in dataObj.paths['serviceGroupsOrder'])
  {
    var serviceGroup = dataObj.paths['serviceGroupsOrder'][serviceGroupIndex];
    html += '    <option value="' + serviceGroup + '">' + serviceGroup + (!dataObj.paths[serviceGroup].active ? ' *' : '') + '</option>\n';
  }

  html += '  </select>\n';
  html += '  <br><sup>*</sup> represents a <b>disabled</b> serviceGroup.<br>\n';
  html += '  <br>\n';
  html += '  <input type="button" name="edit" value="Edit" onClick="processEditServiceGroupClick();">&nbsp;\n';
  html += '  <input type="button" name="delete" value="Delete" onClick="processDeleteServiceGroupClick();">&nbsp;\n';
  if (dataObj.paths['serviceGroupsOrder'].length > 1)
  {
    html += '  <input type="button" name="move" value="Move" onClick="processMoveServiceGroupClick();">\n';
  }
  html += '  <br><br>\n';
  html += '  <input type="button" name="create" value="Create" onClick="createServiceGroupsEntryScreen(\'\');">\n';
  html += '  </center>\n';

  content.innerHTML = html;
}

function displayServiceGroupEntryEditable(name, prefixString, rowSpan)
{
  var output = '';
  try {
    var info = getLookupEntry('serviceGroup', name);
  }
  catch (e) {
    return 'Error getting info! ' + e.message;
  }
  var handler = '';
  var result = info.handler.match(/^(.*onChange=".+)(")(.*)$/);
  if (result != null)
  {
    handler = result[1] + '; entryChangesMade=true;"' + result[3];
  }
  else
  {
    handler = info.handler + ' onChange="entryChangesMade=true;"';
  }

  // create a table to hold the name and user interaction parts in.
  output += '    <td width="50%" valign="top"' + (arguments.length == 3 ? ' rowspan="' + rowSpan + '"' : '') + '>\n';  // start the td we are in.
  if (arguments.length >= 2)
  {
    if (prefixString.length > 0)
    {
      output += '      <center><b>' + prefixString + '</b></center>\n';
    }
  }
  output += '      <table width="100%" border="0" cellpadding="0" cellspacing="2">\n';
  output += '        <tr>\n';
  output += '          <td align="right" width="50%" valign="top"><a href="#" onClick="displayHelp(\'serviceGroup\', \'' + name + '\'); return false;">' + info.displayString + '</a>&nbsp;</td>\n';
  output += '          <td align="left" width="50%">';

  if (info.type == "select")
  {
    output += '            <select name="' + name + '"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n';
  }
  if (info.type == "bool")
  {
    output += '            <input type="checkbox" name="' + name + '" value="true"' + handler + ' />\n';
  }
  if (info.type == "text")
  {
    output += '            <input type="text" name="' + name + '" value=""' + handler + ' onKeyPress="entryChangesMade=true;">\n';
  }
  if (info.type == "textarea")
  {
    output += '            <textarea name="' + name + '"' + handler + ' onKeyPress="entryChangesMade=true;"></textarea>\n';
  }
  if (info.type == "multiselect")
  {
    output += '            <select name="' + name + '" multiple="multiple" size="5"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n<br>\n';
    output += '            ' + selectAllButton(name) + '<br>\n';
    output += '            ' + toggleSelectButton(name) + '\n';
  }
  if (info.type == "mixed")
  {
    output += '            <select name="' + name + '"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n';
    output += '            <input type="text" name="' + name + 'Value" value="" size="3" maxlength="3" onChange="entryChangesMade=true;" onKeyPress="entryChangesMade=true;">\n';
  }

  output += '          </td>\n';
  output += '        </tr>\n';
  output += '      </table>\n';
  output += '    </td>\n';  // close the td we are in.

  return output;
}

function displayPathEntryEditable(name, prefixString, colSpan, center)
{
  var output = '';
  try {
    var info = getLookupEntry('paths', name);
  }
  catch (e) {
    return 'Error getting info! ' + e.message;
  }
  var handler = '';
  var result = info.handler.match(/^(.*onChange=".+)(")(.*)$/);
  if (result != null)
  {
    handler = result[1] + '; entryChangesMade=true;"' + result[3];
  }
  else
  {
    handler = info.handler + ' onChange="entryChangesMade=true;"';
  }

  // create a table to hold the name and user interaction parts in.
  output += '    <td width="' + (arguments.length != 3 ? '50%' : '100%') + '" valign="top"' + (arguments.length >= 3 ? ' colspan="' + colSpan + '"' : '') + '>\n';  // start the td we are in.
  output += '      <table width="' + (arguments.length != 3 ? '100%' : '50%') + '" border="0" cellpadding="0" cellspacing="2">\n';
  if (arguments.length >= 2)
  {
    if (prefixString.length > 0)
    {
      output += '        <tr>\n';
      output += '          <td align="center" colspan="2"><b>' + prefixString + '</b></td>\n';
      output += '        </tr>\n';
    }
  }
  output += '        <tr>\n';
  output += '          <td align="right" width="50%" valign="top"><a href="#" onClick="displayHelp(\'paths\', \'' + name + '\'); return false;">' + info.displayString + '</a>&nbsp;</td>\n';
  output += '          <td align="left" width="50%">';

  if (info.type == "select" || info.type == "select+clear")
  {
    output += '            <select name="' + name + '"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n';

    if (info.type == "select+clear")
    {
      output += '            <input type="button" name="clear_' + name + '" value="Clear" onClick="document.configF.' + name + '.selectedIndex=-1; entryChangesMade=true;">\n';
    }
  }
  if (info.type == "bool")
  {
    output += '            <input type="checkbox" name="' + name + '" value="true"' + handler + ' />\n';
  }
  if (info.type == "text")
  {
    output += '            <input type="text" name="' + name + '" value=""' + handler + ' onKeyPress="entryChangesMade=true;">\n';
  }
  if (info.type == "textarea")
  {
    output += '            <textarea name="' + name + '"' + handler + ' onKeyPress="entryChangesMade=true;"></textarea>\n';
  }
  if (info.type == "multiselect")
  {
    output += '            <select name="' + name + '" multiple="multiple" size="5"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n<br>\n';
    output += '            ' + selectAllButton(name) + '<br>\n';
    output += '            ' + toggleSelectButton(name) + '\n';
  }
  if (info.type == "mixed")
  {
    output += '            <select name="' + name + '"' + handler + '>\n';
    // the options string is actually the function to call since this may be dynamic.
    var func = /^\s*function/;
    if (func.test(info.options))
    {
      output += info.options();
    }
    else
    {
      output += info.options;
    }
    output += '            </select>\n';
    output += '            <input type="text" name="' + name + 'Value" value="" size="3" maxlength="3"' + handler + ' onKeyPress="entryChangesMade=true;">\n';
  }

  output += '          </td>\n';
  output += '        </tr>\n';
  output += '      </table>\n';
  output += '    </td>\n';  // close the td we are in.

  return output;
}

function displayPathScreen(mode, path, index)
{
  var content = getContent();
  var config = document.configF;
  var entry = (index == -1 ? new createServiceGroupInfoEntry("","","",[],[],0,0,0,"","","","",0,"",0,0,"","",[],1,"") : tmpServiceGroupEntry['entry'][path][index]);

  displayStatus((mode == "Create" ? "Creat" : mode) + "ing Service Group <b>" + tmpServiceGroupEntry['origServiceGroupName'] + "</b>'s Path <b>" + path + "</b>...", "info");

  pathChangesMade = entryChangesMade;

  // build up the form html to display the port mapping info in.
  var html = "";
  html += '  <center><b>' + (mode == "Create" ? "Creat" : mode) + 'ing Service Group <i>' + tmpServiceGroupEntry['origServiceGroupName'] + '</i>\'s Path <i>' + path + '</i></b><br>\n';
  html += '  <table border="0" width="75%" cellpadding="0" cellspacing="2">\n';
  html += '    <tr>\n';
  html += displayPathEntryEditable('type');
  html += displayPathEntryEditable('active');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayPathEntryEditable('i');
  html += displayPathEntryEditable('o');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayPathEntryEditable('limitFrom');
  html += displayPathEntryEditable('limitTo');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayPathEntryEditable('action');
  html += displayPathEntryEditable('ip');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += '      <td colspan="2">\n';
  html += '        <table border="1" width="100%" cellpadding="0" cellspacing="0">\n';
  html += '          <tr>\n';
  html += '            <td>\n';
  html += '              <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '                <tr>\n';
  html += '                  <td colspan="2" align="center"><b>Redirect</b></td>\n';
  html += '                </tr>\n';
  html += '                <tr>\n';
  html += displayPathEntryEditable('redirectPort', '', 2);
  html += '                </tr>\n';
  html += '              </table>\n';
  html += '            </td>\n';
  html += '          </tr>\n';
  html += '        </table>\n';
  html += '      </td>\n';
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += '      <td colspan="2">\n';
  html += '        <table border="1" width="100%" cellpadding="0" cellspacing="0">\n';
  html += '          <tr>\n';
  html += '            <td>\n';
  html += '              <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '                <tr>\n';
  html += '                  <td colspan="2" align="center"><b>DNAT</b></td>\n';
  html += '                </tr>\n';
  html += '                <tr>\n';
  html += displayPathEntryEditable('dnatDestination');
  html += displayPathEntryEditable('dnatPort');
  html += '                </tr>\n';
  html += '                <tr>\n';
  html += displayPathEntryEditable('dnatZone');
  html += displayPathEntryEditable('dnatInterface');
  html += '                </tr>\n';
  html += '              </table>\n';
  html += '            </td>\n';
  html += '          </tr>\n';
  html += '        </table>\n';
  html += '      </td>\n';
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += '      <td colspan="2">\n';
  html += '        <table border="1" width="100%" cellpadding="0" cellspacing="0">\n';
  html += '          <tr>\n';
  html += '            <td>\n';
  html += '              <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '                <tr>\n';
  html += '                  <td colspan="2" align="center"><b>Logging</b></td>\n';
  html += '                </tr>\n';
  html += '                <tr>\n';
  html += displayPathEntryEditable('log', '', 2);
  html += '                </tr>\n';
  html += '                <tr>\n';
  html += displayPathEntryEditable('logComment');
  html += displayPathEntryEditable('logLevel');
  html += '                </tr>\n';
  html += '              </table>\n';
  html += '            </td>\n';
  html += '          </tr>\n';
  html += '        </table>\n';
  html += '      </td>\n';
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayPathEntryEditable('service');
  html += displayPathEntryEditable('comment');
  html += '    </tr>\n';
  html += '  </table>\n';
  html += '  <br>\n';
  html += '  <input type="button" name="update" value="' + (mode == "Edit" ? "Update" : mode) + '" onClick="updateServiceGroupEntryInfoEntry(\'' + mode + '\', \'' + path + '\', \'' + index + '\');">';
  html += '  <input type="button" name="cancel" value="Cancel" onClick="entryChangesMade=pathChangesMade; displayServiceGroupsEntryScreen(tmpServiceGroupEntry[\'mode\'], tmpServiceGroupEntry[\'origServiceGroupName\']);"></center>\n';

  content.innerHTML = html;

  // initialize the settings.
  config.type.selectedIndex = (path.length > 0 ? pathTypeOptions[path] : -1);
  config.comment.value = entry.comment;
  config.active.checked = entry.active;
  if (path.length > 0)
  {
    // initialize the in/out entries and the actions being taken.
    if(bothPaths.test(path))
    {
      togglePathInterfaceFields('i', false, path);
      togglePathInterfaceFields('o', false, path);
      selectPathInterface(config.i, entry.i);
      selectPathInterface(config.o, entry.o);
      selectPathHosts(config.limitFrom, entry.limitFrom);
      selectPathHosts(config.limitTo, entry.limitTo);
      config.i.focus();
    }
    else
    {
      if (incomingPaths.test(path))
      {
        togglePathInterfaceFields('i', false, path);
        togglePathInterfaceFields('o', true, path);
        selectPathInterface(config.i, entry.i);
        selectPathHosts(config.limitFrom, entry.limitFrom);
        config.i.focus();
      }
      else
      {
        if (outgoingPaths.test(path))
        {
          togglePathInterfaceFields('i', true, path);
          togglePathInterfaceFields('o', false, path);
          selectPathInterface(config.o, entry.o);
          selectPathHosts(config.limitTo, entry.limitTo);
          config.o.focus();
        }
      }
    }
  }
  else
  {
    togglePathInterfaceFields('i', true, path);
    togglePathInterfaceFields('o', true, path);
  }

  selectPathHosts(config.ip, [entry.ip]);

  // select the action value
  config.action.selectedIndex = pathActionOptions['ACCEPT'];
  // update the redirect, dnat and log sections.
  if (entry.redirect)
  {
    togglePathRedirectFields(false);
    config.action.selectedIndex = pathActionOptions['REDIRECT'];
    config.redirectPort.value = entry.redirectPort;
  }
  else
  {
    togglePathRedirectFields(true);
  }
  if (entry.dnat)
  {
    togglePathDNATFields(false);
    config.action.selectedIndex = pathActionOptions['DNAT'];
    selectPathHosts(config.dnatDestination, [entry.dnatDestination]);
    config.dnatPort.value = entry.dnatPort;
    config.dnatZone.selectedIndex = pathDNATZoneOptions[entry.dnatZone];
    togglePathDNATFields(false);
    selectPathInterface(config.dnatInterface, entry.dnatInterface);
  }
  else
  {
    togglePathDNATFields(true);
    config.dnatDestination.selectedIndex = -1;
    config.dnatZone.selectedIndex = -1;
    config.dnatInterface.selectedIndex = -1;
  }
  if (entry.reject)
  {
    config.action.selectedIndex = pathActionOptions['REJECT'];
  }
  if (entry.queue)
  {
    config.action.selectedIndex = pathActionOptions['QUEUE'];
  }
  if (entry.log)
  {
    config.log.checked = true;
    togglePathLogFields(false);
    config.logComment.value = entry.logComment;
    config.logLevel.selectedIndex = logLevelOptions[entry.logLevel];
  }
  else
  {
    config.log.checked = false;
    togglePathLogFields(false);
  }
  if (entry.dropNoLog)
  {
    config.action.selectedIndex = pathActionOptions['DROP No LOG'];
    config.log.checked = false;
    togglePathLogFields(true);
  }
  selectPathService(entry.service);
}

function editPathEntry(path, index)
{
  // make sure we preserve the serviceGroup name and comment incase it changed.
  var config = document.configF;
  tmpServiceGroupEntry['name'] = config.name.value;
  tmpServiceGroupEntry['entry'].comment = config.comment.value;
  tmpServiceGroupEntry['entry'].active = config.active.checked;

  displayPathScreen('Edit', path, index);
}

function deletePathEntry(path, index)
{
  // make sure we preserve the serviceGroup name and comment incase it changed.
  var config = document.configF;
  tmpServiceGroupEntry['name'] = config.name.value;
  tmpServiceGroupEntry['entry'].comment = config.comment.value;
  tmpServiceGroupEntry['entry'].active = config.active.checked;

  var result = confirm("Are you sure you want to Delete Path '" + path + "', entry '" + index + "'?");
  if (result == false)
  {
    return;
  }

  // verify the entry exists.
  if (tmpServiceGroupEntry['entry'][path][index] == undefined)
  {
    alert("Path '" + path + "' with index = '" + index + "' does not exist to Delete!");
  }
  // delete the entry.
  else
  {
    tmpServiceGroupEntry['entry'][path].splice(index, 1);
    if (numHashEntries(tmpServiceGroupEntry['entry'][path]) == 0)
    {
      delete tmpServiceGroupEntry['entry'][path];

      // find and remove the zonePathOrder entry.
      for (var pathIndex in tmpServiceGroupEntry['entry'].zonePathOrder)
      {
        if (tmpServiceGroupEntry['entry'].zonePathOrder[pathIndex] == path)
        {
          tmpServiceGroupEntry['entry'].zonePathOrder.splice(pathIndex, 1);
        }
      }
    }
    tmpServiceGroupEntry['changed'] = true;
    entryChangesMade = true;
    displayServiceGroupsEntryScreen(tmpServiceGroupEntry['mode'], tmpServiceGroupEntry['origServiceGroupName']);
  }
}

function createPathScreen()
{
  // make sure we preserve the serviceGroup name and comment incase it changed.
  var config = document.configF;
  tmpServiceGroupEntry['name'] = config.name.value;
  tmpServiceGroupEntry['entry'].comment = config.comment.value;
  tmpServiceGroupEntry['entry'].active = config.active.checked;

  displayPathScreen('Create', '', -1);
}

function getSelectedEntries(entry, type)
{
  var entries = new Array();

  for (var i=0; i < entry.options.length; i++)
  {
    if (entry.options[i].selected)
    {
      if (type == "host")
      {
        entries.push(availableNetworksMappingsReverse[entry.options[i].value]);
      }
      else if (type == "service")
      {
        entries.push(entry.options[i].value);
      }
    }
  }

  return entries;
}

function getSelectEntry(entry)
{
  var result = "";
  if (entry.selectedIndex != -1)
  {
    result = entry.options[entry.selectedIndex].value;
  }
  return result;
}

function updateServiceGroupEntryInfoEntry(mode, path, index)
{
  var config = document.configF;
  if (config.type.selectedIndex == -1)
  {
    alert("You must select a path type!");
    return;
  }
  var newPath = config.type.options[config.type.selectedIndex].value;
  var dnatDestination = getSelectedEntries(config.dnatDestination, 'host');
  var action = getSelectEntry(config.action);
  var ip = getSelectedEntries(config.ip, 'host');
  var info = new createServiceGroupInfoEntry(getSelectedEntries(config.i, 'service'),
                                             (ip.length == 1 ? ip[0].toString() : ''),
                                             getSelectedEntries(config.o, 'service'),
                                             getSelectedEntries(config.limitFrom, 'host'),
                                             getSelectedEntries(config.limitTo, 'host'),
                                             (action == "REJECT" ? 1 : 0),
                                             (action == "DROP No LOG" ? 1 : 0),
                                             (action == "DNAT" ? 1 : 0),
                                             dnatDestination[0],
                                             getSelectEntry(config.dnatZone),
                                             getSelectEntry(config.dnatInterface),
                                             config.dnatPort.value,
                                             (action == "REDIRECT" ? 1 : 0),
                                             config.redirectPort.value,
                                             (action == "QUEUE" ? 1 : 0),
                                             config.log.checked,
                                             config.logComment.value,
                                             getSelectEntry(config.logLevel),
                                             getSelectedEntries(config.service, 'service'),
                                             config.active.checked,
                                             config.comment.value);

  // make sure the interfaces are valid.
  if (handleInterfaceChange(config.i, newPath) == false || handleInterfaceChange(config.o, newPath) == false)
  {
    return;
  }
  // make sure they selected a service.
  if (info.service.length == 0)
  {
    alert("You must select a service!");
    return;
  }
  if (mode == "Create")
  {
    if (tmpServiceGroupEntry['entry'][newPath] == undefined)
    {
      tmpServiceGroupEntry['entry'].zonePathOrder.push(newPath);
      tmpServiceGroupEntry['entry'][newPath] = new Array();
    }
    tmpServiceGroupEntry['entry'][newPath].push(info);
  }
  else
  {
    if (path == newPath)
    {
      // update the index.
      tmpServiceGroupEntry['entry'][path][index] = info;
    }
    else
    {
      // we have to remove the old entry and insert into the new path.
      // verify the entry exists.
      if (tmpServiceGroupEntry['entry'][path][index] == undefined)
      {
        alert("Path '" + path + "' with index = '" + index + "' does not exist to Delete (on update of '" + newPath + "')!");
      }
      // delete the entry.
      else
      {
        tmpServiceGroupEntry['entry'][path].splice(index, 1);
        if (numHashEntries(tmpServiceGroupEntry['entry'][path]) == 0)
        {
          delete tmpServiceGroupEntry['entry'][path];

          // find and remove the zonePathOrder entry.
          for (var pathIndex in tmpServiceGroupEntry['entry'].zonePathOrder)
          {
            if (tmpServiceGroupEntry['entry'].zonePathOrder[pathIndex] == path)
            {
              tmpServiceGroupEntry['entry'].zonePathOrder.splice(pathIndex, 1);
            }
          }
        }
      }

      // now we append the rule to the newPath entry, creating if needed.
      if (tmpServiceGroupEntry['entry'][newPath] == undefined)
      {
        tmpServiceGroupEntry['entry'].zonePathOrder.push(newPath);
        tmpServiceGroupEntry['entry'][newPath] = new Array();
      }
      tmpServiceGroupEntry['entry'][newPath].push(info);
    }
  }
  tmpServiceGroupEntry['changed'] = true;
  entryChangesMade = true;
  displayServiceGroupsEntryScreen(tmpServiceGroupEntry['mode'], tmpServiceGroupEntry['origServiceGroupName']);
}

function displayLimitInfo(info)
{
  var networks = dataObj.networks;
  var result = "";

  if (info.length == 0)
  {
    result = "<b>ALL</b>";
  }

  for (var i in info)
  {
    if (result.length > 0)
    {
      result += ", ";
    }
    var comment = networks[info[i]].comment;
    result += (comment.length > 0 ? '<span title="' + comment + '">' : '') + fixupOutput(networks[info[i]].host) + (comment.length > 0 ? '</span>' : '');
  }

  return result;
}

function displayActionInfo(info)
{
  var networks = dataObj.networks;
  var result = "";

  if (info.redirect)
  {
    result += '<span title="Port: ' + info.redirectPort + '">Redirect</span>';
  }
  if (info.reject)
  {
    result += "Reject";
  }
  if (info.dropNoLog)
  {
    result += "Drop No Log";
  }
  if (info.dnat)
  {
    result += '<span title="Host: ' + networks[info.dnatDestination].host + ', Port: ' + info.dnatPort + ', Zone: ' + info.dnatZone + ', Interface: ' + info.dnatInterface + '">DNAT</span>';
  }
  if (info.queue)
  {
    result += "QUEUE";
  }
  if (result.length == 0)
  {
    result += "ACCEPT";
  }
  if (info.log)
  {
    result += ' + <span title="Comment: ' + info.logComment + ', Level: ' + info.logLevel + '">Log</span>';
  }

  return result;
}

function displayServiceInfo(info)
{
  var result = "";

  for (var serviceIndex in info.service)
  {
    var service = info.service[serviceIndex];
    var serviceInfo = dataObj.services[service];
    result += '<span title="';
    for (var index in serviceInfo.infos)
    {
      var entry = serviceInfo.infos[index];
      if (entry.proto == 'tcp' || entry.proto == 'udp')
      {
        result += (entry.dport != '' ? 'DPT: ' + entry.dport + '/' + entry.proto : '');
        result += (entry.dport != '' && entry.sport != '' ? ', ' : '');
        result += (entry.sport != '' ? 'SPT: ' + entry.sport + '/' + entry.proto : '');
      }
      else if (entry.proto == 'icmp')
      {
        result += 'ICMP: ' + (entry.icmpType != '' ? entry.icmpType : 'All icmp traffic');
      }
      else if (entry.proto != '')
      {
        result += 'Protocol: ' + entry.proto;
      }
      else
      {
        result += 'All protocols and tcp/udp/icmp traffic';
      }
      result += (index < serviceInfo.infos.length - 1 ? ' ; ' : '');
    }
    result += '">' + fixupOutput(service) + '</span>';
    result += (serviceIndex < info.service.length - 1 ? ', ' : '');
  }

  return result;
}

function movePathUp(path, index)
{
  var paths = tmpServiceGroupEntry['entry'][path];
  var info = paths[index];

  if (index == 0)
  {
    // loop around to bottom of array
    info = paths.shift();
    paths.push(info);
  }
  else
  {
    // move up by one.
    var temp = paths.splice(index-1, 2);
    temp.reverse();
    if (paths.length > 0)
    {
      paths.splice(index-1,0,temp[0],temp[1]);
    }
    else
    {
      paths = temp;
    }
  }

  tmpServiceGroupEntry['entry'][path] = paths;

  tmpServiceGroupEntry['changed'] = true;
  entryChangesMade = true;
  displayServiceGroupsEntryScreen(tmpServiceGroupEntry['mode'], tmpServiceGroupEntry['origServiceGroupName']);
}

function movePathDown(path, index)
{
  var paths = tmpServiceGroupEntry['entry'][path];
  var newIndex;
  var info = paths[index];

  if (index == numHashEntries(paths) - 1)
  {
    // loop around to top of array
    info = paths.pop();
    paths.unshift(info);
  }
  else
  {
    // move down by one.
    var temp = paths.splice(index, 2);
    temp.reverse();
    if (paths.length > 0)
    {
      paths.splice(index,0,temp[0],temp[1]);
    }
    else
    {
      paths = temp;
    }
  }

  tmpServiceGroupEntry['entry'][path] = paths;

  tmpServiceGroupEntry['changed'] = true;
  entryChangesMade = true;
  displayServiceGroupsEntryScreen(tmpServiceGroupEntry['mode'], tmpServiceGroupEntry['origServiceGroupName']);
}

function displayServiceGroupEntry(path, index, color)
{
  var output = "";
  var networks = dataObj.networks;
  var mode = tmpServiceGroupEntry['mode'];
  var info = tmpServiceGroupEntry['entry'][path][index];
  var move = (numHashEntries(tmpServiceGroupEntry['entry'][path]) > 1 ? true : false);

  output += '          <tr bgColor="' + color + '" style="border: solid 1pt;">\n';
  output += '            <td align="center" style="border: solid 1pt;"><a href="#" onClick="editPathEntry(\'' + path + '\', ' + index + '); return false;" title="Edit path">E</a></td>\n';
  if (move)
  {
    output += '            <td align="center" style="border: solid 1pt;"><a href="#" onClick="movePathUp(\''+path+'\',\''+index+'\'); return false;" title="Move Path Up">^</a>/<a href="#" onClick="movePathDown(\''+path+'\',\''+index+'\'); return false;" title="Move Path Down">v</a></td>\n';
  }
  else
  {
    output += '            <td align="center" style="border: solid 1pt;">&nbsp;</td>\n';
  }
  output += '            <td align="center" style="border: solid 1pt;">' + fixupOutput(info.i.join()) + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + fixupOutput(info.o.join()) + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + (info.ip.length > 0 ? fixupOutput(networks[info.ip].host) : "&nbsp;") + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + displayLimitInfo(info.limitFrom) + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + displayLimitInfo(info.limitTo) + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + displayActionInfo(info) + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + displayServiceInfo(info) + '</td>\n';
  output += '            <td align="center" style="border: solid 1pt;">' + (info.active ? "true" : "false") + '</td>\n';
  output += '            <td align="center"><a href="#" onClick="deletePathEntry(\'' + path + '\', ' + index + '); return false;" title="Delete Path">D</a></td>\n';
  output += '          </tr>\n';
  return output;
}

function displayServiceGroupsEntryScreen(mode, service)
{
  var content = getContent();
  var config = document.configF;
  var entry = tmpServiceGroupEntry['entry'];

  displayStatus(mode + (mode != "Create" ? "ing" : "") + " ServiceGroup" + (mode != "Create" ? " '" + fixupOutput(service) + "'" : "") + "...<br>Edit, Delete and Create <b>Paths</b> that direct the firewall to handle packets that would like to go through your firewall, to the firewall or leave the firewall.  <b>E</b> is the Edit link for a Path, <b>D</b> is the Delete link for a Path.  If you have defined multiple <b>Paths</b> per path type, then you can order them using the <b>^</b> and <b>v</b> links.", "info");

  // build up the form html to display the dataObj.services info in.
  var html = "";
  html += '  <input type="hidden" name="origServiceGroupName" value="' + tmpServiceGroupEntry['origServiceGroupName'] + '">\n';
  html += '  <center><b>' + mode + ' ServiceGroup' + (mode != "Create" ? ' \'' + fixupOutput(service) + '\'' : '') + '</b><br>\n';
  html += '  <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '    <tr>\n';
  html += displayServiceGroupEntryEditable('name');
  html += displayServiceGroupEntryEditable('comment');
  html += '    </tr>\n';
  html += displayServiceGroupEntryEditable('active');
  html += '      <td>&nbsp;</td>\n';
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += '      <td colspan="2" align="center"><input type="button" name="createPathEntry" value="Create Path" onClick="createPathScreen()"><br><br>Paths:</td>\n';
  html += '    </tr>\n';
  if (numHashEntries(entry.zonePathOrder) > 0)
  {
    html += '    <tr>\n';
    html += '      <td colspan="2">\n';
    html += '        <table style="border-collapse: collapse; border: hidden;" cellpadding="2" cellspacing="2" width="100%">\n';
    html += '          <tr>\n';
    html += '            <th>&nbsp;</th><th align="center">move</th><th align="center">in</th><th align="center">out</th><th align="center">ip</th><th align="center">source</th><th align="center">dest</th><th align="center"> action </th><th align="center">service</th><th align="center"> active</th><th>&nbsp;</th>\n';
    html += '          </tr>\n';
    var counter = true;
    for (var pathIndex=0; pathIndex < numHashEntries(entry.zonePathOrder); pathIndex++)
    {
      var path = entry.zonePathOrder[pathIndex];
      html += '          <tr>\n';
      html += '            <td colspan="11"><b>' + fixupOutput(path) + '</b></td>\n';
      html += '          </tr>\n';
      for (var index=0; index < numHashEntries(entry[path]); index++)
      {
        var color = (counter ? "lightblue" : "white");
        html += displayServiceGroupEntry(path, index, color);
        counter = !counter;
      }
    }
    html += '        </table>\n';
    html += '      </td>\n';
    html += '    </tr>\n';
  }
  else
  {
    html += '    <tr>\n';
    html += '      <td align="center" colspan="10">No Paths currently defined...</td>\n';
    html += '    </tr>\n';
  }
  html += '  </table>\n';
  html += '  <br>\n';
  html += '  <input type="button" name="update" value="Update" onClick="updateServiceGroupsEntry(\'' + mode + '\', \'' + service + '\');"></center>\n';

  content.innerHTML = html;

  // initialize the settings.
  config.name.value = tmpServiceGroupEntry['name'];
  config.comment.value = entry.comment;
  config.active.checked = entry.active;
}

function moveServiceGroupsEntry(service)
{
  // display the screen to allow the user to move the selected serviceGroup entry to the top, bottom or before another
  // serviceGroup entry.
  var content = getContent();
  var config = document.configF;

  displayStatus("Move ServiceGroup '" + fixupOutput(service) + "'...", "info");

  // build up the form html to display the move serviceGroup info in.
  var html = "";
  html += '  <center><b>Move ServiceGroup \'' + fixupOutput(service) + '\'</b><br>\n';
  html += '  <br>\n';
  html += '  <input type="button" name="top" value="to Top" onClick="moveServiceGroupEntry(\'top\', \'' + service + '\');"><br>\n';
  html += '  <input type="button" name="top" value="Before" onClick="moveServiceGroupEntry(\'before\', \'' + service + '\');">';
  html += '  <select name="serviceGroup">\n';

  for (var serviceGroupIndex in dataObj.paths['serviceGroupsOrder'])
  {
    var serviceGroup = dataObj.paths['serviceGroupsOrder'][serviceGroupIndex];
    html += '    <option value="' + serviceGroup + '">' + serviceGroup + (!dataObj.paths[serviceGroup].active ? ' *' : '') + '</option>\n';
  }

  html += '  </select>\n';
  html += '  <br>\n';
  html += '  <input type="button" name="top" value="to Bottom" onClick="moveServiceGroupEntry(\'bottom\', \'' + service + '\');"><br>\n';
  html += '  </center>\n';

  content.innerHTML = html;
}

function moveServiceGroupEntry(location, service)
{
  var config = document.configF;
  var serviceGroups = dataObj.paths['serviceGroupsOrder'];
  var index = -1;

  // first find out where the service is in the serviceGroupsOrder array.
  for (var serviceGroupIndex=0; serviceGroupIndex < serviceGroups.length; serviceGroupIndex++)
  {
    var serviceGroup = serviceGroups[serviceGroupIndex];
    if (serviceGroup == service)
    {
      index = serviceGroupIndex;
      break;
    }
  }

  // alert ("Service = '" + service + "' is at index = '" + index + "', location = '" + location + "'");

  if (index == -1)
  {
    alert("Could not find serviceGroup = '" + service + "'!");
    return;
  }

  if (location == "top")
  {
    var tmp = serviceGroups.splice(index, 1);
    serviceGroups.unshift(tmp);
  }
  else if (location == "bottom")
  {
    var tmp = serviceGroups.splice(index, 1);
    serviceGroups.push(tmp);
  }
  else if (location == "before")
  {
    // find the location they are referring to.
    var serviceGroup = config.serviceGroup.options[config.serviceGroup.selectedIndex].value;

    // now find the selected serviceGroup index.
    var index2 = -1;
    for (var i=0; i < serviceGroups.length; i++)
    {
      var s = serviceGroups[i];
      if (s == serviceGroup)
      {
        index2 = i;
        break;
      }
    }

    if (index2 == -1)
    {
      alert("Could not find serviceGroup = '" + serviceGroup + "' to move " + location + "!");
      return;
    }

    var offset = -1;
    // now calculate the offset after we have removed the current service.
    if (index2 < index)
    {
      if (location == "before")
      {
        offset = index2;
      }
    }
    else if (index2 >= index)
    {
      // we have to subtract 1 from index2 as it is now moved up the array.
      if (location == "before")
      {
        offset = index2 - 1;
      }
    }

    // alert ("Service = '" + service + "' is at index = '" + index + "', location = '" + location + "', Service2 = '" + serviceGroup + "' is at index = '" + index2 + "', moving to offset = '" + offset + "'");

    if (offset == -1)
    {
      alert("offset = -1!");
      return;
    }

    var tmp = serviceGroups.splice(index, 1);
    serviceGroups.splice(offset, 0, tmp[0]);
  }
  else
  {
    alert("Invalid location '" + location + "'!  serviceGroup = '" + service + "'");
    return;
  }

  dataObj.paths['serviceGroupsOrder'] = serviceGroups;
  changesMade = true;
  entryChangesMade = false;

  // redisplay the serviceGroup screen.
  displayServiceGroups();
}

function editServiceGroupsEntry(service)
{
  tmpServiceGroupEntry['name'] = service;
  tmpServiceGroupEntry['origServiceGroupName'] = service;
  tmpServiceGroupEntry['entry'] = cloneArray(dataObj.paths[service]);  // try to make a copy.
  tmpServiceGroupEntry['mode'] = 'Edit';
  tmpServiceGroupEntry['changed'] = false;
  displayServiceGroupsEntryScreen('Edit', service);
}

function deleteServiceGroupsEntry(service)
{
  var paths = dataObj.paths;
  var pathCount = 0;
  var result = confirm("Are you sure you want to Delete ServiceGroup " + service + "'?");
  if (result == false)
  {
    return;
  }

  // verify the entry exists.
  if (paths[service] == undefined)
  {
    alert("ServiceGroup " + service + "' does not exist to Delete!");
  }
  // delete the entry.
  else
  {
    delete paths[service];
    // find the service and remove from the serviceGroupsOrder array.
    for (var serviceIndex in paths['serviceGroupsOrder'])
    {
      if (paths['serviceGroupsOrder'][serviceIndex] == service)
      {
        paths['serviceGroupsOrder'].splice(serviceIndex, 1);
        break;
      }
    }
    changesMade = true;
    entryChangesMade = false;
  }

  // redisplay the ServiceGroups screen.
  displayServiceGroups();
}

function createServiceGroupsEntryScreen(service)
{
  tmpServiceGroupEntry['name'] = '';
  tmpServiceGroupEntry['origServiceGroupName'] = '';
  tmpServiceGroupEntry['entry'] = new Object;
  tmpServiceGroupEntry['entry'].comment = '';
  tmpServiceGroupEntry['entry'].active = 1;
  tmpServiceGroupEntry['entry'].zonePathOrder = new Array();
  tmpServiceGroupEntry['mode'] = 'Create';
  tmpServiceGroupEntry['changed'] = false;
  displayServiceGroupsEntryScreen('Create', '');
}

function updateServiceGroupsEntry(mode, service)
{
  var config = document.configF;
  var paths = dataObj.paths;
  var updateServiceGroupName = false;
  var entry = tmpServiceGroupEntry['entry'];

  if (mode != "Create")
  {
    if (config.origServiceGroupName.value != config.name.value)
    {
      // make sure this is not a duplicate serviceGroup.
      for (var i in paths['serviceGroupsOrder'])
      {
        if (paths['serviceGroupsOrder'][i] == config.name.value)
        {
          alert("You already have a serviceGroup defined = '" + config.name.value + "'!");
          config.name.focus();
          return;
        }
      }
      updateServiceGroupName = true;
    }
    entry.name = config.name.value;
  }
  else
  {
    // make sure this is not a duplicate service.
    for (var i in paths['serviceGroupsOrder'])
    {
      if (paths['serviceGroupsOrder'][i] == config.name.value)
      {
        alert("You already have a serviceGroup defined = '" + config.name.value + "'!");
        config.name.focus();
        return;
      }
    }
    entry.name = config.name.value;
  }
  var oldServiceGroupName = config.origServiceGroupName.value;

  if (config.name.value.length == 0)
  {
    alert("You must specify the serviceGroup name!");
    config.name.focus();
    return;
  }

  if (config.comment.value != entry.comment)
  {
    entryChangesMade = true;
    entry.comment = config.comment.value;
  }

  if (config.active.checked != entry.active)
  {
    entryChangesMade = true;
    entry.active = config.active.checked;
  }

  // make sure we have at least 1 paths entry.
  if (numHashEntries(entry.zonePathOrder) == 0)
  {
    alert("You must have at least 1 Path created!");
    pathsError = true;
  }

  if (pathsError)
  {
    // reset the error flag.
    pathsError = false;
  }
  else
  {
    if (mode == "Create")
    {
      paths['serviceGroupsOrder'].push(entry.name);
    }
    else if (updateServiceGroupName)
    {
      for (var i=0; i < paths['serviceGroupsOrder'].length; i++)
      {
        if (paths['serviceGroupsOrder'][i] == oldServiceGroupName)
        {
          paths['serviceGroupsOrder'][i] = entry.name;
          delete paths[oldServiceGroupName];
          break;
        }
      }
    }
    if (entryChangesMade || updateServiceGroupName || tmpServiceGroupEntry['changed'])
    {
      changesMade = true;
      entryChangesMade = false;
      paths[entry.name] = cloneArray(entry);
    }
    displayServiceGroups();
  }
}
